/**
 * Copyright (C) 2008 Atlassian
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.atlassian.theplugin.commons.cfg;

import com.atlassian.theplugin.commons.ServerType;

public class JiraServerCfg extends ServerCfg {
    private static final int HASHCODE_MAGIC = 31;
    private boolean dontUseBasicAuth;

    public JiraServerCfg(final String name, final ServerIdImpl serverId, boolean dontUseBasicAuth) {
		super(true, name, serverId);
        this.dontUseBasicAuth = dontUseBasicAuth;
    }

	public JiraServerCfg(final JiraServerCfg other) {
		super(other);
        this.dontUseBasicAuth = other.dontUseBasicAuth;
	}

	public JiraServerCfg(boolean enabled, String name, ServerIdImpl serverId, boolean dontUseBasicAuth) {
		super(enabled, name, serverId);
        this.dontUseBasicAuth = dontUseBasicAuth;
    }

    public JiraServerCfg(boolean enabled, String name, String url, ServerIdImpl serverId, boolean dontUseBasicAuth) {
        super(enabled, name, url, serverId);
        this.dontUseBasicAuth = dontUseBasicAuth;
    }

    @Override
	public ServerType getServerType() {
		return ServerType.JIRA_SERVER;
	}

	@Override
	public JiraServerCfg getClone() {
		return new JiraServerCfg(this);
	}

    public boolean isDontUseBasicAuth() {
        return dontUseBasicAuth;
    }

    public void setDontUseBasicAuth(boolean dontUseBasicAuth) {
        this.dontUseBasicAuth = dontUseBasicAuth;
    }

    @Override
    public boolean equals(final Object o) {
        if (!super.equals(o)) {
            return false;
        }

        if (this == o) {
            return true;
        }
        if (!(o instanceof JiraServerCfg)) {
            return false;
        }

        final JiraServerCfg that = (JiraServerCfg) o;

        return dontUseBasicAuth == that.dontUseBasicAuth;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = HASHCODE_MAGIC * result + (dontUseBasicAuth ? 1 : 0);
        return result;
    }

}
