/**
 * Copyright (C) 2008 Atlassian
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.atlassian.theplugin.commons.crucible.api.model;

import org.jetbrains.annotations.Nullable;

import java.io.Serializable;
import java.util.Collection;

//
//* @parm allowedReviewers is != null for Cru 2.0 or greater
//*
public class CrucibleProject implements Serializable {
	private final String id;
	private final String key;
	private final String name;

    private final Collection<String> allowedReviewers;
    private static final int HASH_INT = 31;

	public CrucibleProject(String id, String key, String name) {
		this(id, key, name, null);
	}

	public CrucibleProject(String id, String key, String name, Collection<String> allowedReviewers) {
		this.id = id;
		this.key = key;
		this.name = name;
        this.allowedReviewers = allowedReviewers;
    }

    @Nullable
    public Collection<String> getAllowedReviewers() {
        return allowedReviewers;
    }

    public String getId() {
		return id;
	}

	public String getKey() {
		return key;
	}

	public String getName() {
		return name;
	}

	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		CrucibleProject that = (CrucibleProject) o;

		if (id != null ? !id.equals(that.id) : that.id != null) {
			return false;
		}
		if (key != null ? !key.equals(that.key) : that.key != null) {
			return false;
		}
		if (name != null ? !name.equals(that.name) : that.name != null) {
			return false;
		}

		return true;
	}

	public int hashCode() {
		int result;
		result = (id != null ? id.hashCode() : 0);
		result = HASH_INT * result + (key != null ? key.hashCode() : 0);
		result = HASH_INT * result + (name != null ? name.hashCode() : 0);
        result = HASH_INT * result + (allowedReviewers != null ? allowedReviewers.hashCode() : 0);
		return result;
	}
}
